#!/usr/bin/env ruby

# == Synopsis
#
# nwt
#
# == Usage
#

$LOAD_PATH << File.join(File.dirname(__FILE__), "../../../")

require "rdcl/link/serial_dock_link.rb"
require "rdcl/link/app_cmds/app_cmd_install_package.rb"
require "rdcl/link/dock_cmds/dock_cmd_factory.rb"
require "rdcl/utils/des.rb"
require "yaml"
require "getoptlong"
# require "rdoc/usage"

include RDCL

Thread.abort_on_exception = true

class NWT < Actor
  
  attr_accessor :settings
  attr_accessor :dock
  attr_accessor :opts
  attr_accessor :cmd
  attr_accessor :flags
  attr_accessor :newton_stores
  
  def initialize
    super()
    @flags = {}
    @settings = YAML::load(File.new(Platform.settings_file))

    @opts = GetoptLong.new(
      ["--help", "-h", GetoptLong::NO_ARGUMENT],
      ["--verbose", "-v", GetoptLong::NO_ARGUMENT],
      ["--install", "-i", GetoptLong::REQUIRED_ARGUMENT],
      ["--uninstall", "-u", GetoptLong::REQUIRED_ARGUMENT],
      ["--info", "-n", GetoptLong::NO_ARGUMENT],
      ["--quiet", "-q", GetoptLong::NO_ARGUMENT],
      ["--test", "-t", GetoptLong::NO_ARGUMENT],
      ["--log", GetoptLong::NO_ARGUMENT],
      ["--query", GetoptLong::REQUIRED_ARGUMENT]
    )
    
    def uninstall_transition(state, action)
      return state
    end
    
    def install_transition(state, action)
      case action.command
      when AppCmd::CONNECTED then
        log "Installing #{@arg}."
        @dock.receive(AppCmdInstallPackage.new(@arg))
      when AppCmd::PACKAGE_INSTALLED then
        @dock.receive(AppCmd.new(AppCmd::DISCONNECT))
      end
      return state
    end
    
    def test_transition(state, action)
      return state
    end
    
    def query_transition(state, action)
      case action.command
      when AppCmd::CONNECTED then
        @dock.receive(AppCmd.new(AppCmd::GET_STORE_NAMES))

      when AppCmd::STORE_NAMES then
        puts "Store names: \n#{action.data.to_s}"
        @newton_stores = action.data.to_ruby
        @dock.receive(AppCmd.new(AppCmd::SET_CURRENT_STORE, :default))

      when AppCmd::STORE_SELECTED then
        @dock.receive(AppCmd.new(AppCmd::GET_SOUP_NAMES))

      when AppCmd::SOUP_NAMES then
        puts "Soup names: \n#{action.data.to_s}"
        @dock.receive(AppCmd.new(AppCmd::SET_CURRENT_SOUP, @arg))

      when AppCmd::SOUP_SELECTED then
        @dock.receive(AppCmd.new(AppCmd::QUERY_SOUP, {
            :querySpec => {
              :indexPath => :timestamp
            }
          }))

      end
      return state
    end
    
    def info_transition(state, action)
      case action.command
      when AppCmd::CONNECTED then
        log "Getting device information."
        @dock.receive(AppCmd.new(AppCmd::GET_STORE_NAMES))
      when AppCmd::STORE_NAMES then
        puts "Store names: \n#{action.data.to_s}"
        @newton_stores = action.data.to_ruby
        @dock.receive(AppCmd.new(AppCmd::SET_CURRENT_STORE, :default))
      when AppCmd::STORE_SELECTED then
        @dock.receive(AppCmd.new(AppCmd::GET_SOUP_NAMES))
      when AppCmd::SOUP_NAMES then
        puts "Soup names: \n#{action.data.to_s}"
        @dock.receive(AppCmd.new(AppCmd::GET_APP_LIST))
      when AppCmd::APP_LIST then
        puts "Applications: \n#{action.data.to_s}"
      end
      return state
    end
    
    def log_action(state, action)
      if @flags[:verbose] then puts "<<<\n" + action.to_yaml end
    end
    
    def log(string)
      if not @flags[:quiet] then puts(string) end
    end
    
    transition do |state, action|
      log_action(state, action)
      state = send("#{@cmd}_transition", state, action)
      case action.command
      when AppCmd::CONNECTED then
        log "Connected."
      when AppCmd::DISCONNECTED then
        log "Disconnected."
        exit
      end
      state
    end
  end

  def parse_opts
    continue = true
    @opts.each do |opt, arg|
      case opt
      when "--help" then continue = nil
      when "--install" then @cmd = "install"; @arg = arg;
      when "--uninstall" then @cmd = "uninstall"; @arg = arg;
      when "--test" then @cmd = "test"; @arg = arg;
      when "--query" then @cmd = "query"; @arg = arg;
      when "--info" then @cmd = "info"
      when "--verbose" then @flags[:verbose] = true
      when "--quiet" then @flags[:quiet] = true
      when "--log" then @flags[:log] = true
      end
    end
    return continue
  end

  def setup_dock
    @dock = SerialDockLink.new(settings, @flags[:log])
    @dock.application = self
  end

  def start
    run
    @dock.run
    log "Waiting for connection on #{@settings['serial_port']}..."
    @thread.join
  end
  
end

app = NWT.new
if app.parse_opts
  app.setup_dock
  app.start
else
#  RDoc::usage
end
