require "rdcl/link/dock_modules/dock_module.rb"
require "rdcl/link/app_cmds/app_cmd_default_store.rb"
require "rdcl/link/app_cmds/app_cmd_store_names.rb"
require "rdcl/link/app_cmds/app_cmd_soup_names.rb"

module RDCL

  class StorageDockModule < DockModule
    
    def initialize(dock_link)
      super(dock_link, :storage_list_dock_module)
      
      @log_transitions = nil

      init_automaton :idle, [:idle]
      transition do |s, a|
        new_state = case [s, a]
        when [:idle, AppCmd::GET_DEFAULT_STORE] then start_get_default_store; :get_default_store
        when [:get_default_store, DockCmd::DEFAULT_STORE] then finish_get_default_store; :idle

        when [:idle, AppCmd::GET_STORE_NAMES] then start_get_store_names; :get_store_names
        when [:get_store_names, DockCmd::STORE_NAMES] then finish_get_store_names; :idle
        
        when [:idle, AppCmd::QUERY_SOUP] then query_soup; :soup_query
        when [:soup_query, DockCmd::LONG_DATA] then set_cursor_id; :idle

        when [:idle, AppCmd::SET_CURRENT_STORE] then set_current_store; :set_current_store
        when [:set_current_store, DockCmd::RESULT] then store_selected; :idle

        when [:idle, AppCmd::SET_CURRENT_SOUP] then set_current_soup; :set_current_soup
        when [:set_current_soup, DockCmd::RESULT] then soup_selected; :idle

        when [:idle, AppCmd::GET_SOUP_NAMES] then get_soup_names; :get_soup_names
        when [:get_soup_names, DockCmd::SOUP_NAMES] then finish_get_soup_names; :idle

        else nil
        end

        new_state = s if not new_state
        new_state
      end
    end

    def start_get_default_store
      dock_write(DockCmdGetDefaultStore.new)
    end

    def finish_get_default_store
      @command.data.slice!(0)
      obj = NSObjectFactory.new.from_nsof_factory(@command.data)
      app_receive(AppCmdDefaultStore.new(obj))
    end

    def start_get_store_names
      dock_write(DockCmdGetStoreNames.new)
    end

    def finish_get_store_names
      @command.data.slice!(0)
      obj = NSObjectFactory.new.from_nsof_factory(@command.data)
      app_receive(AppCmdStoreNames.new(obj))
    end
    
    def query_soup
      b = 2.chr + @command.data.to_nsof
      dock_write(DockCmd.new(DockCmd::QUERY, b))
    end
    
    def set_cursor_id
      puts @command
    end

    def set_current_soup
      dock_write(DockCmdSetCurrentSoup.new(@command.data))
    end
    
    def soup_selected
      app_receive(AppCmd.new(AppCmd::SOUP_SELECTED, @command.result))
    end

    def set_current_store
      if @command.data == :default
        dock_write(DockCmd.new(DockCmd::SET_STORE_TO_DEFAULT))
      end
    end
    
    def store_selected
      app_receive(AppCmd.new(AppCmd::STORE_SELECTED, @command.result))
    end
    
    def get_soup_names
      dock_write(DockCmd.new(DockCmd::GET_SOUP_NAMES))
    end

    def finish_get_soup_names
      @command.data.slice!(0)
      obj = NSObjectFactory.new.from_nsof_factory(@command.data)
      app_receive(AppCmdSoupNames.new(obj))
    end
    
  end
  
end