require "rdcl/nsobjects/ns_object.rb"
require "rdcl/nsobjects/ns_plainarray.rb"
require "rdcl/nsobjects/ns_array.rb"
require "rdcl/nsobjects/ns_string.rb"
require "rdcl/nsobjects/ns_frame.rb"
require "rdcl/nsobjects/ns_integer.rb"
require "rdcl/nsobjects/ns_symbol.rb"
require "rdcl/nsobjects/ns_precendent.rb"
require "rdcl/nsobjects/ns_true.rb"
require "rdcl/nsobjects/ns_magic_pointer.rb"
require "rdcl/nsobjects/ns_nil.rb"
require "rdcl/nsof/nsof.rb"

module RDCL
  
  class NSObjectFactory
    
    attr_accessor :objects

    NSOBJECT_MAP = [
      [NSOF::IMMEDIATE, :immediate],
      [NSOF::CHARACTER, :NSCharacter],
      [NSOF::UNICODECHARACTER, :NSUnicodeCharacter],
      [NSOF::BINARYOBJECT, nil],
      [NSOF::ARRAY, :NSArray],
      [NSOF::PLAINARRAY, :NSPlainArray],
      [NSOF::FRAME, :NSFrame],
      [NSOF::SYMBOL, :NSSymbol],
      [NSOF::STRING, :NSString],
      [NSOF::PRECEDENT, :NSPrecedent],
      [NSOF::NIL, :NSNil],
      [NSOF::SMALLRECT, nil],
      [NSOF::LARGEBINARY, nil],
    ]
    
    def initialize
      @objects = []
    end

    def from_nsof_factory(b)
      r = nil
      class_symbol = nil
      c = b.unpack("C")
      NSOBJECT_MAP.each do |obj|
        if obj[0] == c[0]
          class_symbol = obj[1]
        end
      end
      if not class_symbol
        puts "Error decoding code #{c[0]}"
      end
      if class_symbol == :immediate
        b.slice!(0)
        v = NSOF::decode_xlong(b)
        if v & 3 == 0
          r = NSInteger.new(v >> 2)
        elsif v & 0xf == 6
          r = NSCharacter.new((v >> 4) & 0xffff)
        elsif v == 0x1a
          r = NSTrue.new
        elsif v == 2
          r = NSNil.new
        elsif v & 3 == 3
          r = NSMagicPointer.new(v)
        end
      else
        r = RDCL.const_get(class_symbol).new
        r.from_nsof(b, self)
      end

      if class_symbol == :NSPrecedent
        r = @objects[r.value]
      end

      return r
    end

  end
  
end
